/*
 *   kdiff3 - Text Diff And Merge Tool
 *   This file only: Copyright (C) 2002  Joachim Eibl, joachim.eibl@gmx.de
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

/***************************************************************************
 * $Log$                                                                   *
 ***************************************************************************/

#ifndef OPTION_DIALOG_H
#define OPTION_DIALOG_H

class QCheckBox;
class QLabel;
class QLineEdit;
class KColorButton;
class KFontChooser;
class KConfig;

#include <kdialogbase.h>
#include <qstrlist.h>
#include <list>

class OptionItem;
class KKeyDialog;

class OptionDialog : public KDialogBase
{
   Q_OBJECT

public:

    OptionDialog( QWidget *parent = 0, char *name = 0 );
    ~OptionDialog( void );

    // These are the results of the option dialog.
    QFont m_font;
    bool m_bItalicForDeltas;

    QColor m_fgColor;
    QColor m_bgColor;
    QColor m_diffBgColor;
    QColor m_colorA;
    QColor m_colorB;
    QColor m_colorC;
    QColor m_colorForConflict;
    QColor m_currentRangeBgColor;
    QColor m_currentRangeDiffBgColor;

    bool m_bReplaceTabs;
    bool m_bAutoIndentation;
    int  m_tabSize;
    bool m_bAutoCopySelection;

    bool m_bIgnoreTrivialMatches;
    int  m_maxSearchLength;
    bool m_bPreserveCarriageReturn;
    bool m_bUseExternalDiff;
    bool m_bTryHard;
    bool m_bShowWhiteSpace;
    bool m_bShowLineNumbers;

    bool m_bIgnoreWhiteSpace;
    bool m_bUpCase;
    bool m_bIgnoreNumbers;
    QString m_PreProcessorCmd;
    QString m_LineMatchingPreProcessorCmd;

    bool m_bAutoAdvance;
    int  m_autoAdvanceDelay;

    QStrList m_recentAFiles;
    QStrList m_recentBFiles;
    QStrList m_recentCFiles;

    QStrList m_recentOutputFiles;

    // Directory Merge options
    bool m_bDmSyncMode;
    bool m_bDmRecursiveDirs;
    bool m_bDmFollowFileLinks;
    bool m_bDmFollowDirLinks;
    bool m_bDmFindHidden;
    bool m_bDmCreateBakFiles;
    bool m_bDmTrustDate;
    bool m_bDmCopyNewer;
    bool m_bDmShowOnlyDeltas;    
    QString m_DmFilePattern;
    QString m_DmFileAntiPattern;

    void saveOptions(KConfig* config);
    void readOptions(KConfig* config);

    void setState(); // Must be called before calling exec();

    void addOptionItem(OptionItem*);
    KKeyDialog* m_pKeyDialog;
protected slots:
    virtual void slotDefault( void );
    virtual void slotOk( void );
    virtual void slotApply( void );
    virtual void slotHelp( void );

private:
    void resetToDefaults();
    
    std::list<OptionItem*> m_optionItemList;
    
    // FontConfigDlg
    KFontChooser *m_fontChooser;

private:
    void setupFontPage();
    void setupColorPage();
    void setupEditPage();
    void setupDiffPage();
    void setupDirectoryMergePage();
    void setupKeysPage();
};



#endif







